@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdproject.c 13846 2014-12-28 21:46:54Z pwessel $
+ *	$Id: grdproject.c 18110 2017-05-03 01:29:16Z pwessel $
  *
- *	Copyright (c) 1991-2015 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -26,35 +26,32 @@
  * Ver:		5 API
  */
 
+#include "gmt_dev.h"
+
 #define THIS_MODULE_NAME	"grdproject"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Forward and inverse map transformation of grids"
-
-#include "gmt_dev.h"
-
-#define GMT_PROG_OPTIONS "-JRVnr" GMT_OPT("FS")
+#define THIS_MODULE_KEYS	"<G{,GG}"
+#define THIS_MODULE_NEEDS	"J"
+#define THIS_MODULE_OPTIONS "-JRVnr" GMT_OPT("S")
 
 struct GRDPROJECT_CTRL {
 	struct In {	/* Input grid */
 		bool active;
 		char *file;
 	} In;
-	struct A {	/* -A[k|m|n|i|c|p] */
-		bool active;
-		char unit;
-	} A;
 	struct C {	/* -C[<dx/dy>] */
 		bool active;
 		double easting, northing;
 	} C;
-	struct D {	/* -Ddx[/dy] */
-		bool active;
-		double inc[2];
-	} D;
 	struct E {	/* -E<dpi> */
 		bool active;
 		int dpi;
 	} E;
+	struct F {	/* -F[k|m|n|i|c|p] */
+		bool active;
+		char unit;
+	} F;
 	struct G {	/* -G */
 		bool active;
 		char *file;
@@ -68,55 +65,55 @@ struct GRDPROJECT_CTRL {
 	} M;
 };
 
-void *New_grdproject_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
+GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct GRDPROJECT_CTRL *C;
 	
-	C = GMT_memory (GMT, NULL, 1, struct GRDPROJECT_CTRL);
+	C = gmt_M_memory (GMT, NULL, 1, struct GRDPROJECT_CTRL);
 	
 	/* Initialize values whose defaults are not 0/false/NULL */
 		
 	return (C);
 }
 
-void Free_grdproject_Ctrl (struct GMT_CTRL *GMT, struct GRDPROJECT_CTRL *C) {	/* Deallocate control structure */
+GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDPROJECT_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
-	if (C->In.file) free (C->In.file);	
-	if (C->G.file) free (C->G.file);	
-	GMT_free (GMT, C);	
+	gmt_M_str_free (C->In.file);	
+	gmt_M_str_free (C->G.file);	
+	gmt_M_free (GMT, C);	
 }
 
-int GMT_grdproject_usage (struct GMTAPI_CTRL *API, int level)
-{
-	GMT_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
+	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdproject <ingrid> -G<outgrid> %s [-A[%s|%s]] [-C[<dx>/<dy>]]\n", GMT_J_OPT, GMT_LEN_UNITS2_DISPLAY, GMT_DIM_UNITS_DISPLAY);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-D%s] [-E<dpi>] [-I] [-M%s]\n", GMT_inc_OPT, GMT_DIM_UNITS_DISPLAY);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s]\n\t[%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_n_OPT, GMT_r_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: grdproject <ingrid> -G<outgrid> %s [-C[<dx>/<dy>]] [-D%s]\n",
+		GMT_J_OPT, GMT_inc_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-E<dpi>] [-F[%s|%s]] [-I] [-M%s] [%s] [%s]\n", GMT_LEN_UNITS2_DISPLAY,
+		GMT_DIM_UNITS_DISPLAY, GMT_DIM_UNITS_DISPLAY, GMT_Rgeo_OPT, GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s]\n\n", GMT_n_OPT, GMT_r_OPT);
 
-	if (level == GMT_SYNOPSIS) return (EXIT_FAILURE);
+	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	GMT_Message (API, GMT_TIME_NONE, "\t<ingrid> is data set to be projected.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-G Set name of output grid\n");
 	GMT_Option (API, "J");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-A Force projected values to be in actual distance units [Default uses the given map scale].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Specify unit by appending e (meter), f (foot) k (km), M (mile), n (nautical mile), u (survey foot),\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   or i (inch), c (cm), or p (points) [e].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-C Coordinates are relative to projection center [Default is relative to lower left corner].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Optionally append dx/dy to add (or subtract if -I) (i.e., false easting & northing) [0/0].\n");
-	GMT_inc_syntax (API->GMT, 'D', 0);
+	gmt_inc_syntax (API->GMT, 'D', 0);
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Set dpi for output grid.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-F Force projected values to be in actual distance units [Default uses the given map scale].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Specify unit by appending e (meter), f (foot) k (km), M (mile), n (nautical mile), u (survey foot),\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   or i (inch), c (cm), or p (points) [e].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Inverse transformation from rectangular to geographical.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-M Temporarily reset PROJ_LENGTH_UNIT to be c (cm), i (inch), or p (point).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Cannot be used if -A is set.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Cannot be used if -F is set.\n");
 	GMT_Option (API, "R");
 	GMT_Option (API, "V,n,r,.");
 
-	return (EXIT_FAILURE);
+	return (GMT_MODULE_USAGE);
 }
 
-int GMT_grdproject_parse (struct GMT_CTRL *GMT, struct GRDPROJECT_CTRL *Ctrl, struct GMT_OPTION *options)
-{
+GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDPROJECT_CTRL *Ctrl, struct GMT_OPTION *options) {
 	/* This parses the options provided to grdproject and sets parameters in CTRL.
 	 * Any GMT common options will override values set previously by other commands.
 	 * It also replaces any file names specified as input or output with the data ID
@@ -134,7 +131,7 @@ int GMT_grdproject_parse (struct GMT_CTRL *GMT, struct GRDPROJECT_CTRL *Ctrl, st
 
 			case '<':	/* Input files */
 				if (n_files++ > 0) break;
-				if ((Ctrl->In.active = GMT_check_filearg (GMT, '<', opt->arg, GMT_IN, GMT_IS_GRID)))
+				if ((Ctrl->In.active = gmt_check_filearg (GMT, '<', opt->arg, GMT_IN, GMT_IS_GRID)))
 					Ctrl->In.file = strdup (opt->arg);
 				else
 					n_errors++;
@@ -142,31 +139,35 @@ int GMT_grdproject_parse (struct GMT_CTRL *GMT, struct GRDPROJECT_CTRL *Ctrl, st
 
 			/* Processes program-specific parameters */
 
-			case 'A':	/* Force meters */
-				Ctrl->A.active = true;
-				Ctrl->A.unit = opt->arg[0];
-				break;
 			case 'C':	/* Coordinates relative to origin */
 				Ctrl->C.active = true;
 				if (opt->arg[0]) 	/* Also gave shifts */
-					n_errors += GMT_check_condition (GMT, sscanf (opt->arg, "%lf/%lf", &Ctrl->C.easting, &Ctrl->C.northing) != 2,
+					n_errors += gmt_M_check_condition (GMT, sscanf (opt->arg, "%lf/%lf", &Ctrl->C.easting, &Ctrl->C.northing) != 2,
 						 "Syntax error: Expected -C[<false_easting>/<false_northing>]\n");
 				break;
 			case 'D':	/* Grid spacings */
-				Ctrl->D.active = true;
-				if (GMT_getinc (GMT, opt->arg, Ctrl->D.inc)) {
-					GMT_inc_syntax (GMT, 'D', 1);
-					n_errors++;
-				}
+				n_errors += gmt_parse_inc_option (GMT, 'D', opt->arg);
 				break;
 			case 'E':	/* Set dpi of grid */
 				Ctrl->E.active = true;
 				sval = atoi (opt->arg);
-				n_errors += GMT_check_condition (GMT, sval <= 0, "Syntax error -E option: Must specify positive dpi\n");
+				n_errors += gmt_M_check_condition (GMT, sval <= 0, "Syntax error -E option: Must specify positive dpi\n");
 				Ctrl->E.dpi = sval;
 				break;
+			case 'A':	/* Old Force specific unit option */
+				if (gmt_M_compat_check (GMT, 5))	/* Honor old -A[<unit>] option */
+					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -A is deprecated; use -F instead.\n");
+				else {
+					n_errors += gmt_default_error (GMT, opt->option);
+					break;
+				}
+				/* Fall through on purpose to get -F */
+			case 'F':	/* Force specific unit */
+				Ctrl->F.active = true;
+				Ctrl->F.unit = opt->arg[0];
+				break;
 			case 'G':	/* Output file */
-				if ((Ctrl->G.active = GMT_check_filearg (GMT, 'G', opt->arg, GMT_OUT, GMT_IS_GRID)))
+				if ((Ctrl->G.active = gmt_check_filearg (GMT, 'G', opt->arg, GMT_OUT, GMT_IS_GRID)))
 					Ctrl->G.file = strdup (opt->arg);
 				else
 					n_errors++;
@@ -177,47 +178,43 @@ int GMT_grdproject_parse (struct GMT_CTRL *GMT, struct GRDPROJECT_CTRL *Ctrl, st
 			case 'M':	/* Directly specify units */
 				Ctrl->M.active = true;
 				Ctrl->M.unit = opt->arg[0];
-				n_errors += GMT_check_condition (GMT, !Ctrl->M.unit,
+				n_errors += gmt_M_check_condition (GMT, !Ctrl->M.unit,
 							"Syntax error -M option: projected measure unit must be one of 'c', i', or 'p'\n");
 				break;
-			case 'N':	/* GMT4 Backwards compatible.  nx/ny can now be set with -D */
-				if (GMT_compat_check (GMT, 4)) {
+			case 'N':	/* GMT4 Backwards compatible.  n_columns/n_rows can now be set with -D */
+				if (gmt_M_compat_check (GMT, 4)) {
 					GMT_Report (API, GMT_MSG_COMPAT, "Warning: -N option is deprecated; use -D instead.\n");
-					Ctrl->D.active = true;
 					sscanf (opt->arg, "%d/%d", &ii, &jj);
 					if (jj == 0) jj = ii;
 					sprintf (format, "%d+/%d+", ii, jj);
-					if (GMT_getinc (GMT, format, Ctrl->D.inc)) {
-						GMT_inc_syntax (GMT, 'D', 1);
-						n_errors++;
-					}
+					n_errors += gmt_parse_inc_option (GMT, 'D', format);
 				}
 				else
-					n_errors += GMT_default_error (GMT, opt->option);
+					n_errors += gmt_default_error (GMT, opt->option);
 				break;
 			default:	/* Report bad options */
-				n_errors += GMT_default_error (GMT, opt->option);
+				n_errors += gmt_default_error (GMT, opt->option);
 				break;
 		}
 	}
 
-	GMT_check_lattice (GMT, Ctrl->D.inc, &GMT->common.r.registration, &Ctrl->D.active);
+	//gmt_check_lattice (GMT, Ctrl->D.inc, &GMT->common.R.registration, &Ctrl->D.active);
 
-	n_errors += GMT_check_condition (GMT, !Ctrl->In.file, "Syntax error: Must specify input file\n");
-	n_errors += GMT_check_condition (GMT, !Ctrl->G.file, "Syntax error -G option: Must specify output file\n");
-	n_errors += GMT_check_condition (GMT, !GMT->common.J.active, "Syntax error: Must specify a map projection with the -J option\n");
-	n_errors += GMT_check_condition (GMT, (Ctrl->M.active + Ctrl->A.active) == 2, "Syntax error: Can specify only one of -A and -M\n");
-	n_errors += GMT_check_condition (GMT, (Ctrl->D.active + Ctrl->E.active) > 1, "Syntax error: Must specify only one of -D or -E\n");
-	n_errors += GMT_check_condition (GMT, Ctrl->D.active && (Ctrl->D.inc[GMT_X] <= 0.0 || Ctrl->D.inc[GMT_Y] < 0.0), "Syntax error -D option: Must specify positive increment(s)\n");
+	n_errors += gmt_M_check_condition (GMT, !Ctrl->In.file, "Syntax error: Must specify input file\n");
+	n_errors += gmt_M_check_condition (GMT, !Ctrl->G.file, "Syntax error -G option: Must specify output file\n");
+	n_errors += gmt_M_check_condition (GMT, !GMT->common.J.active, "Syntax error: Must specify a map projection with the -J option\n");
+	n_errors += gmt_M_check_condition (GMT, (Ctrl->M.active + Ctrl->F.active) == 2, "Syntax error: Can specify only one of -F and -M\n");
+	n_errors += gmt_M_check_condition (GMT, (GMT->common.R.active[ISET] + Ctrl->E.active) > 1, "Syntax error: Must specify only one of -D or -E\n");
+	n_errors += gmt_M_check_condition (GMT, GMT->common.R.active[ISET] && (GMT->common.R.inc[GMT_X] <= 0.0 || GMT->common.R.inc[GMT_Y] < 0.0),
+	                                 "Syntax error -D option: Must specify positive increment(s)\n");
 
-	return (n_errors ? GMT_PARSE_ERROR : GMT_OK);
+	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
-#define bailout(code) {GMT_Free_Options (mode); return (code);}
-#define Return(code) {Free_grdproject_Ctrl (GMT, Ctrl); GMT_end_module (GMT, GMT_cpy); bailout (code);}
+#define bailout(code) {gmt_M_free_options (mode); return (code);}
+#define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
-int GMT_grdproject (void *V_API, int mode, void *args)
-{
+int GMT_grdproject (void *V_API, int mode, void *args) {
 	bool set_n = false, shift_xy = false;
 	unsigned int use_nx = 0, use_ny = 0, offset, k, unit = 0;
 	int error = 0;
@@ -231,59 +228,59 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 	struct GRDPROJECT_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
-	struct GMTAPI_CTRL *API = GMT_get_API_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
+	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 
 	/*----------------------- Standard module initialization and parsing ----------------------*/
 
 	if (API == NULL) return (GMT_NOT_A_SESSION);
-	if (mode == GMT_MODULE_PURPOSE) return (GMT_grdproject_usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
+	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (GMT_grdproject_usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (GMT_grdproject_usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
+	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
 
 	/* Parse the command-line arguments */
 
-	GMT = GMT_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
-	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
-	Ctrl = New_grdproject_Ctrl (GMT);	/* Allocate and initialize a new control structure */
-	if ((error = GMT_grdproject_parse (GMT, Ctrl, options))) Return (error);
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
+	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
+	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the grdproject main code ----------------------------*/
 
 	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input grid\n");
-	GMT_set_pad (GMT, 2U);	/* Ensure space for BCs in case an API passed pad == 0 */
-	if ((Ctrl->D.active + Ctrl->E.active) == 0) set_n = true;
-	if (Ctrl->M.active) GMT_err_fail (GMT, GMT_set_measure_unit (GMT, Ctrl->M.unit), "-M");
+	gmt_set_pad (GMT, 2U);	/* Ensure space for BCs in case an API passed pad == 0 */
+	if ((GMT->common.R.active[ISET] + Ctrl->E.active) == 0) set_n = true;
+	if (Ctrl->M.active) gmt_M_err_fail (GMT, gmt_set_measure_unit (GMT, Ctrl->M.unit), "-M");
 	shift_xy = !(Ctrl->C.easting == 0.0 && Ctrl->C.northing == 0.0);
 	
-	unit = GMT_check_scalingopt (GMT, 'A', Ctrl->A.unit, scale_unit_name);
-	GMT_init_scales (GMT, unit, &fwd_scale, &inv_scale, &inch_to_unit, &unit_to_inch, unit_name);
+	unit = gmt_check_scalingopt (GMT, 'A', Ctrl->F.unit, scale_unit_name);
+	gmt_init_scales (GMT, unit, &fwd_scale, &inv_scale, &inch_to_unit, &unit_to_inch, unit_name);
 
-	if (GMT->common.R.active)	/* Load the w/e/s/n from -R */
-		GMT_memcpy (wesn, GMT->common.R.wesn, 4, double);
+	if (GMT->common.R.active[RSET])	/* Load the w/e/s/n from -R */
+		gmt_M_memcpy (wesn, GMT->common.R.wesn, 4, double);
 	else {	/* If -R was not given we infer the option via the input grid */
 		char opt_R[GMT_BUFSIZ];
 		struct GMT_GRID *G = NULL;
-		if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_HEADER_ONLY, NULL, Ctrl->In.file, NULL)) == NULL) {	/* Get header only */
+		if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->In.file, NULL)) == NULL) {	/* Get header only */
 			Return (API->error);
 		}
-		GMT_memcpy (wesn, G->header->wesn, 4, double);
+		gmt_M_memcpy (wesn, G->header->wesn, 4, double);
 		if (!Ctrl->I.active) {
 			sprintf (opt_R, "%.12f/%.12f/%.12f/%.12f", wesn[XLO], wesn[XHI], wesn[YLO], wesn[YHI]);
-			GMT_parse_common_options (GMT, "R", 'R', opt_R);
-			if (GMT_map_setup (GMT, GMT->common.R.wesn)) Return (GMT_RUNTIME_ERROR);
+			gmt_parse_common_options (GMT, "R", 'R', opt_R);
+			if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, GMT->common.R.wesn), "")) Return (GMT_PROJECTION_ERROR);
 		}
 		else {			/* Do inverse transformation */
-			double x_c, y_c, lon_t, lat_t, ww, ee, ss, nn;
+			double x_c, y_c, lon_t, lat_t, xSW, ySW, xNW, yNW, xNE, yNE, xSE, ySE;
 			/* Obtain a first crude estimation of the good -R */
 			x_c = (wesn[XLO] + wesn[XHI]) / 2.0; 		/* mid point of projected coords */
 			y_c = (wesn[YLO] + wesn[YHI]) / 2.0; 
 			if (GMT->current.proj.projection == GMT_UTM && GMT->current.proj.utm_hemisphere == -1 && y_c > 0) y_c *= -1;
 			if (y_c > 0)
-				GMT_parse_common_options (GMT, "R", 'R', "-180/180/0/80");
+				gmt_parse_common_options (GMT, "R", 'R', "-180/180/0/80");
 			else
-				GMT_parse_common_options (GMT, "R", 'R', "-180/180/-80/0");
+				gmt_parse_common_options (GMT, "R", 'R', "-180/180/-80/0");
 			if (GMT->current.proj.projection == GMT_UTM && GMT->current.proj.utm_hemisphere == -1 && y_c < 0) y_c *= -1;	/* Undo the *-1 (only for the UTM case) */ 
 			if (shift_xy) {
 				x_c -= Ctrl->C.easting;
@@ -295,7 +292,7 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 				y_c *= fwd_scale;
 			}
 
-			if (GMT_map_setup (GMT, GMT->common.R.wesn)) Return (GMT_RUNTIME_ERROR);
+			if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, GMT->common.R.wesn), "")) Return (GMT_PROJECTION_ERROR);
 
 			x_c *= GMT->current.proj.scale[GMT_X];
 			y_c *= GMT->current.proj.scale[GMT_Y];
@@ -304,12 +301,12 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 				x_c += GMT->current.proj.origin[GMT_X];
 				y_c += GMT->current.proj.origin[GMT_Y];
 			}
-			GMT_xy_to_geo (GMT, &lon_t, &lat_t, x_c, y_c);
+			gmt_xy_to_geo (GMT, &lon_t, &lat_t, x_c, y_c);
 			sprintf (opt_R, "%.12f/%.12f/%.12f/%.12f", lon_t-1, lon_t+1, lat_t-1, lat_t+1);
-			if (GMT_is_verbose (GMT, GMT_MSG_VERBOSE)) GMT_Message (API, GMT_TIME_NONE, "First opt_R\t %s\t%g\t%g\n", opt_R, x_c, y_c);
-			GMT->common.R.active = false;	/* We need to reset this to not fall into non-wanted branch deeper down */
-			GMT_parse_common_options (GMT, "R", 'R', opt_R);
-			if (GMT_map_setup (GMT, GMT->common.R.wesn)) Return (GMT_RUNTIME_ERROR);
+			if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) GMT_Message (API, GMT_TIME_NONE, "First opt_R\t %s\t%g\t%g\n", opt_R, x_c, y_c);
+			GMT->common.R.active[RSET] = false;	/* We need to reset this to not fall into non-wanted branch deeper down */
+			gmt_parse_common_options (GMT, "R", 'R', opt_R);
+			if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, GMT->common.R.wesn), "")) Return (GMT_PROJECTION_ERROR);
 
 			/* Finally obtain the good limits */
 			if (shift_xy) {
@@ -326,32 +323,35 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 				wesn[YLO] += GMT->current.proj.origin[GMT_Y];	wesn[YHI] += GMT->current.proj.origin[GMT_Y];
 			}
 
-			GMT_xy_to_geo (GMT, &ww, &ss, wesn[XLO], wesn[YLO]);		/* SW corner */
-			GMT_xy_to_geo (GMT, &ee, &nn, wesn[XHI], wesn[YHI]);		/* NE corner */
-			sprintf (opt_R, "%.12f/%.12f/%.12f/%.12fr", ww, ss, ee, nn);
-			if (GMT_is_verbose (GMT, GMT_MSG_VERBOSE)) GMT_Message (API, GMT_TIME_NONE, "Second opt_R\t %s\n", opt_R);
-			GMT->common.R.active = false;
-			GMT_parse_common_options (GMT, "R", 'R', opt_R);
-			GMT_memcpy (wesn, GMT->common.R.wesn, 4, double);	/* Load up our best wesn setting - it will be used below if -I */
+			gmt_xy_to_geo (GMT, &xSW, &ySW, wesn[XLO], wesn[YLO]);		/* SW corner */
+			gmt_xy_to_geo (GMT, &xNW, &yNW, wesn[XLO], wesn[YHI]);		/* NW corner */
+			gmt_xy_to_geo (GMT, &xNE, &yNE, wesn[XHI], wesn[YHI]);		/* NE corner */
+			gmt_xy_to_geo (GMT, &xSE, &ySE, wesn[XHI], wesn[YLO]);		/* SE corner */
+			sprintf (opt_R, "%.12f/%.12f/%.12f/%.12fr", MIN(xSW, xNW), MIN(ySW, ySE), MAX(xNE, xSE), MAX(yNW, yNE));
+
+			if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) GMT_Message (API, GMT_TIME_NONE, "Second opt_R\t %s\n", opt_R);
+			GMT->common.R.active[RSET] = false;
+			gmt_parse_common_options (GMT, "R", 'R', opt_R);
+			gmt_M_memcpy (wesn, GMT->common.R.wesn, 4, double);	/* Load up our best wesn setting - it will be used below if -I */
 		}
-		if (GMT_Destroy_Data (API, &G) != GMT_OK) {
+		if (GMT_Destroy_Data (API, &G) != GMT_NOERROR) {
 			Return (API->error);
 		}
 	}
 
-	if (GMT_map_setup (GMT, GMT->common.R.wesn)) Return (GMT_RUNTIME_ERROR);
+	if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, GMT->common.R.wesn), "")) Return (GMT_PROJECTION_ERROR);
 
-	if (Ctrl->I.active) {	/* Must flip the column types since in is Cartesian and out is geographic */
-		GMT_set_geographic (GMT, GMT_OUT);	/* Inverse projection expects x,y and gives lon, lat */
-		GMT_set_cartesian (GMT, GMT_IN);
+	if (Ctrl->I.active) {			/* Must flip the column types since in is Cartesian and out is geographic */
+		gmt_set_geographic (GMT, GMT_OUT);	/* Inverse projection expects x,y and gives lon, lat */
+		gmt_set_cartesian (GMT, GMT_IN);
 	}
-	
+
 	xmin = (Ctrl->C.active) ? GMT->current.proj.rect[XLO] - GMT->current.proj.origin[GMT_X] : GMT->current.proj.rect[XLO];
 	xmax = (Ctrl->C.active) ? GMT->current.proj.rect[XHI] - GMT->current.proj.origin[GMT_X] : GMT->current.proj.rect[XHI];
 	ymin = (Ctrl->C.active) ? GMT->current.proj.rect[YLO] - GMT->current.proj.origin[GMT_Y] : GMT->current.proj.rect[YLO];
 	ymax = (Ctrl->C.active) ? GMT->current.proj.rect[YHI] - GMT->current.proj.origin[GMT_Y] : GMT->current.proj.rect[YHI];
-	if (Ctrl->A.active) {	/* Convert to chosen units */
-		strncpy (unit_name, scale_unit_name, GMT_GRID_UNIT_LEN80);
+	if (Ctrl->F.active) {	/* Convert to chosen units */
+		strncpy (unit_name, scale_unit_name, GMT_GRID_UNIT_LEN80-1);
 		xmin /= GMT->current.proj.scale[GMT_X];
 		xmax /= GMT->current.proj.scale[GMT_X];
 		ymin /= GMT->current.proj.scale[GMT_Y];
@@ -376,41 +376,49 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 		ymax += Ctrl->C.northing;
 	}
 
-	sprintf (format, "(%s/%s/%s/%s)", GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
+	sprintf (format, "(%s/%s/%s/%s)", GMT->current.setting.format_float_out, GMT->current.setting.format_float_out,
+	                                  GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
 
 	if (Ctrl->I.active) {	/* Transforming from rectangular projection to geographical */
 
-		/* if (GMT->common.R.oblique) double_swap (s, e); */  /* Got w/s/e/n, make into w/e/s/n */
+		/* if (GMT->common.R.oblique) gmt_M_double_swap (s, e); */  /* Got w/s/e/n, make into w/e/s/n */
 
-		if ((Rect = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_ALL, NULL, Ctrl->In.file, NULL)) == NULL) {
+		if ((Rect = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->In.file, NULL)) == NULL) {
 			Return (API->error);
 		}
 
 		if ((Geo = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_NONE, Rect)) == NULL) Return (API->error);	/* Just to get a header we can change */
 
-		if (GMT_IS_AZIMUTHAL(GMT) && GMT->current.proj.polar) {	/* Watch out for polar cap grids */
-			if (doubleAlmostEqual (GMT->current.proj.pole, -90.0)) {	/* Covers S pole; implies 360 longitude range */
-				wesn[XLO] = -180.0;	wesn[XHI] = +180.0;	wesn[YHI] = MAX(wesn[YLO], wesn[YHI]);	wesn[YLO] = -90.0;
-			}
-			else if (doubleAlmostEqual (GMT->current.proj.pole, +90.0)) {	/* Covers N pole; implies 360 longitude range */
-				wesn[XLO] = -180.0;	wesn[XHI] = +180.0;	wesn[YLO] = MIN(wesn[YLO], wesn[YHI]);	wesn[YHI] = +90.0;
+		if (gmt_M_is_azimuthal(GMT) && GMT->current.proj.polar) {	/* Watch out for polar cap grids */
+			double px, py;
+			/* Get projected pole point and determine if it is inside the grid */
+			gmt_geo_to_xy (GMT, 0.0, GMT->current.proj.pole, &px, &py);	/* Projected coordinates of the relevant pole */
+			if (!gmt_M_y_is_outside (GMT, py,  Rect->header->wesn[YLO], Rect->header->wesn[YHI]) &&
+				!gmt_x_is_outside (GMT, &px,  Rect->header->wesn[XLO], Rect->header->wesn[XHI])) {	/* Not outside both projected x or y-range */
+				wesn[XLO] = -180.0;	wesn[XHI] = +180.0;	/* Need a full 360 longitude range */
+				if (GMT->current.proj.north_pole) {	/* And one of the latitude bounds must extend to the right pole */
+					wesn[YLO] = MIN (wesn[YLO], wesn[YHI]);	wesn[YHI] = +90.0;
+				}
+				else {
+					wesn[YHI] = MAX (wesn[YLO], wesn[YHI]);	wesn[YLO] = -90.0;
+				}
 			}
 		}
-		GMT_memcpy (Geo->header->wesn, wesn, 4, double);
+		gmt_M_memcpy (Geo->header->wesn, wesn, 4, double);
 
 		offset = Rect->header->registration;	/* Same as input */
-		if (GMT->common.r.active) offset = !offset;	/* Toggle */
+		if (GMT->common.R.active[GSET]) offset = !offset;	/* Toggle */
 		if (set_n) {
-			use_nx = Rect->header->nx;
-			use_ny = Rect->header->ny;
+			use_nx = Rect->header->n_columns;
+			use_ny = Rect->header->n_rows;
 		}
-		GMT_err_fail (GMT, GMT_project_init (GMT, Geo->header, Ctrl->D.inc, use_nx, use_ny, Ctrl->E.dpi, offset), Ctrl->G.file);
-		GMT_set_grddim (GMT, Geo->header);
-		if (GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_GRID_DATA_ONLY, NULL, NULL, NULL, 0, 0, Geo) == NULL) Return (API->error);
-		GMT_grd_init (GMT, Geo->header, options, true);
-		GMT_BC_init (GMT, Geo->header);
+		gmt_M_err_fail (GMT, gmt_project_init (GMT, Geo->header, GMT->common.R.inc, use_nx, use_ny, Ctrl->E.dpi, offset), Ctrl->G.file);
+		gmt_set_grddim (GMT, Geo->header);
+		if (GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_DATA_ONLY, NULL, NULL, NULL, 0, 0, Geo) == NULL) Return (API->error);
+		gmt_grd_init (GMT, Geo->header, options, true);
+		gmt_BC_init (GMT, Geo->header);
 
-		if (GMT_is_verbose (GMT, GMT_MSG_VERBOSE)) {
+		if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
 			GMT_Report (API, GMT_MSG_VERBOSE, "Transform ");
 			GMT_Message (API, GMT_TIME_NONE, format, Geo->header->wesn[XLO], Geo->header->wesn[XHI], Geo->header->wesn[YLO], Geo->header->wesn[YHI]);
 			GMT_Message (API, GMT_TIME_NONE, " <-- ");
@@ -418,7 +426,7 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 			GMT_Message (API, GMT_TIME_NONE, " [%s]\n", unit_name);
 		}
 
-		/* Modify input rect header if -A, -C, -M have been set */
+		/* Modify input rect header if -F, -C, -M have been set */
 
 		if (shift_xy) {
 			Rect->header->wesn[XLO] -= Ctrl->C.easting;
@@ -427,7 +435,7 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 			Rect->header->wesn[YHI] -= Ctrl->C.northing;
 
 		}
-		if (Ctrl->A.active) {	/* Convert from 1:1 scale */
+		if (Ctrl->F.active) {	/* Convert from 1:1 scale */
 			if (unit) for (k = 0; k < 4; k++) Rect->header->wesn[k] *= inv_scale;	/* Undo the 1:1 unit used */
 			Rect->header->wesn[XLO] *= GMT->current.proj.scale[GMT_X];
 			Rect->header->wesn[XHI] *= GMT->current.proj.scale[GMT_X];
@@ -443,45 +451,47 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 			Rect->header->wesn[YLO] += GMT->current.proj.origin[GMT_Y];
 			Rect->header->wesn[YHI] += GMT->current.proj.origin[GMT_Y];
 		}
-		GMT_set_grdinc (GMT, Rect->header);	/* Update inc and r_inc given changes to wesn */
-		
+		gmt_set_grdinc (GMT, Rect->header);	/* Update inc and r_inc given changes to wesn */
+
 		sprintf (Geo->header->x_units, "longitude [degrees_east]");
 		sprintf (Geo->header->y_units, "latitude [degrees_north]");
 
-		GMT_grd_project (GMT, Rect, Geo, true);
+		Geo->header->ProjRefPROJ4 = strdup("+proj=longlat +no_defs");	/* HOWEVER, this may be quite incorrect for we are ignoring the DATUM */
 
-		GMT_set_pad (GMT, API->pad);	/* Reset to session default pad before output */
+		gmt_grd_project (GMT, Rect, Geo, true);
+
+		gmt_set_pad (GMT, API->pad);	/* Reset to session default pad before output */
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Geo)) Return (API->error);
-		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_ALL, NULL, Ctrl->G.file, Geo) != GMT_OK) {
+		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->G.file, Geo) != GMT_NOERROR) {
 			Return (API->error);
 		}
 	}
 	else {	/* Forward projection from geographical to rectangular grid */
 
-		if ((Geo = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_ALL, NULL, Ctrl->In.file, NULL)) == NULL) {
+		if ((Geo = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->In.file, NULL)) == NULL) {
 			Return (API->error);
 		}
 
 		if ((Rect = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_NONE, Geo)) == NULL) Return (API->error);	/* Just to get a header we can change */
-		GMT_memcpy (Rect->header->wesn, GMT->current.proj.rect, 4, double);
-		if (Ctrl->A.active) {	/* Convert from 1:1 scale */
+		gmt_M_memcpy (Rect->header->wesn, GMT->current.proj.rect, 4, double);
+		if (Ctrl->F.active) {	/* Convert from 1:1 scale */
 			if (unit) {	/* Undo the 1:1 unit used */
-				Ctrl->D.inc[GMT_X] *= inv_scale;
-				Ctrl->D.inc[GMT_Y] *= inv_scale;
+				GMT->common.R.inc[GMT_X] *= inv_scale;
+				GMT->common.R.inc[GMT_Y] *= inv_scale;
 			}
-			Ctrl->D.inc[GMT_X] *= GMT->current.proj.scale[GMT_X];
-			Ctrl->D.inc[GMT_Y] *= GMT->current.proj.scale[GMT_Y];
+			GMT->common.R.inc[GMT_X] *= GMT->current.proj.scale[GMT_X];
+			GMT->common.R.inc[GMT_Y] *= GMT->current.proj.scale[GMT_Y];
 		}
 		else if (GMT->current.setting.proj_length_unit != GMT_INCH) {	/* Convert from inch to whatever */
-			Ctrl->D.inc[GMT_X] *= unit_to_inch;
-			Ctrl->D.inc[GMT_Y] *= unit_to_inch;
+			GMT->common.R.inc[GMT_X] *= unit_to_inch;
+			GMT->common.R.inc[GMT_Y] *= unit_to_inch;
 		}
 		if (set_n) {
-			use_nx = Geo->header->nx;
-			use_ny = Geo->header->ny;
+			use_nx = Geo->header->n_columns;
+			use_ny = Geo->header->n_rows;
 		}
 
-		if (GMT_is_verbose (GMT, GMT_MSG_VERBOSE)) {
+		if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
 			GMT_Report (API, GMT_MSG_VERBOSE, "Transform ");
 			GMT_Message (API, GMT_TIME_NONE, format, Geo->header->wesn[XLO], Geo->header->wesn[XHI], Geo->header->wesn[YLO], Geo->header->wesn[YHI]);
 			GMT_Message (API, GMT_TIME_NONE, " --> ");
@@ -490,16 +500,16 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 		}
 
 		offset = Geo->header->registration;	/* Same as input */
-		if (GMT->common.r.active) offset = !offset;	/* Toggle */
+		if (GMT->common.R.active[GSET]) offset = !offset;	/* Toggle */
 
-		GMT_err_fail (GMT, GMT_project_init (GMT, Rect->header, Ctrl->D.inc, use_nx, use_ny, Ctrl->E.dpi, offset), Ctrl->G.file);
-		GMT_set_grddim (GMT, Rect->header);
-		if (GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_GRID_DATA_ONLY, NULL, NULL, NULL, 0, 0, Rect) == NULL) Return (API->error);
-		GMT_BC_init (GMT, Rect->header);
-		GMT_grd_project (GMT, Geo, Rect, false);
-		GMT_grd_init (GMT, Rect->header, options, true);
+		gmt_M_err_fail (GMT, gmt_project_init (GMT, Rect->header, GMT->common.R.inc, use_nx, use_ny, Ctrl->E.dpi, offset), Ctrl->G.file);
+		gmt_set_grddim (GMT, Rect->header);
+		if (GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_DATA_ONLY, NULL, NULL, NULL, 0, 0, Rect) == NULL) Return (API->error);
+		gmt_BC_init (GMT, Rect->header);
+		gmt_grd_project (GMT, Geo, Rect, false);
+		gmt_grd_init (GMT, Rect->header, options, true);
 
-		/* Modify output rect header if -A, -C, -M have been set */
+		/* Modify output rect header if -F, -C, -M have been set */
 
 		if (Ctrl->C.active) {	/* Change origin from lower left to projection center */
 			Rect->header->wesn[XLO] -= GMT->current.proj.origin[GMT_X];
@@ -507,7 +517,7 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 			Rect->header->wesn[YLO] -= GMT->current.proj.origin[GMT_Y];
 			Rect->header->wesn[YHI] -= GMT->current.proj.origin[GMT_Y];
 		}
-		if (Ctrl->A.active) {	/* Convert to 1:1 scale */
+		if (Ctrl->F.active) {	/* Convert to 1:1 scale */
 			Rect->header->wesn[XLO] /= GMT->current.proj.scale[GMT_X];
 			Rect->header->wesn[XHI] /= GMT->current.proj.scale[GMT_X];
 			Rect->header->wesn[YLO] /= GMT->current.proj.scale[GMT_Y];
@@ -524,18 +534,21 @@ int GMT_grdproject (void *V_API, int mode, void *args)
 			Rect->header->wesn[YHI] += Ctrl->C.northing;
 
 		}
-		GMT_set_grdinc (GMT, Rect->header);	/* Update inc and r_inc given changes to wesn */
-		strncpy (Rect->header->x_units, unit_name, GMT_GRID_UNIT_LEN80);
-		strncpy (Rect->header->y_units, unit_name, GMT_GRID_UNIT_LEN80);
+		gmt_set_grdinc (GMT, Rect->header);	/* Update inc and r_inc given changes to wesn */
+		strncpy (Rect->header->x_units, unit_name, GMT_GRID_UNIT_LEN80-1);
+		strncpy (Rect->header->y_units, unit_name, GMT_GRID_UNIT_LEN80-1);
+
+		Rect->header->ProjRefPROJ4 = gmt_export2proj4(GMT);	/* Convert the GMT -J<...> into a proj4 string and save it in the header */
 
 		/* rect xy values are here in GMT projected units chosen by user */
 
-		GMT_set_pad (GMT, API->pad);	/* Reset to session default pad before output */
+		gmt_set_pad (GMT, API->pad);	/* Reset to session default pad before output */
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Rect)) Return (API->error);
-		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_ALL, NULL, Ctrl->G.file, Rect) != GMT_OK) {
+		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->G.file, Rect) != GMT_NOERROR) {
 			Return (API->error);
 		}
+		gmt_M_str_free (Rect->header->ProjRefPROJ4);
 	}
 
-	Return (GMT_OK);
+	Return (GMT_NOERROR);
 }
